## See the end of this file for an example
rls <- function(prmRLS)
{
  ## k-step recursive least squares.
  ## Predict y_{t+k} from y_t,y_{t-1},...,xregPrd_{t+k},xreg_t
  ## 
  ## y: the output
  ## k: the steps ahead to predict
  ## lags: the AR lags to include. e.g. c(0,1,2) is an AR(3)
  ## xregPrd: external regressors. Predictions available for t+k at t
  ## xreg: external regressors. Measurements available at same time as y
  ## lambda: forgetting factor.
  ## maxNA: If more observations are missing than maxNA, at a given time, the parameters are not updated and no prediction is carried out.
  ##        note that NA values are counted, not NaN (Note that NaN parameters are not still not updated).
  
  ## The horizon in steps
  k <- prmRLS$k
  ## Number of samples
  n <- length(prmRLS$y)
  ## Set y
  y <- as.numeric(prmRLS$y)

  ##
  ## Form the regressor, start with an intercept, which is removed at the end if not wanted
  X <- rep(1,n)
  namesTheta <- c("intercept")
  ## Add the AR lags
  if( "lags" %in% names(prmRLS) )
    {
      X <- as.matrix(cbind(X, lagMat(prmRLS$y,prmRLS$lags)))
      namesTheta <- c(namesTheta, paste("lag",prmRLS$lags,sep=""))
    }
  ## Add the xreg
  if( "xreg" %in% names(prmRLS) )
    {
      X <- as.matrix(cbind(X,prmRLS$xreg))
      namesTheta <- c(namesTheta, names(prmRLS$xreg))
    }
  ## Add the xregPrd, note that they are lagged ahead to sync correctly when used later
  if( "xregPrd" %in% names(prmRLS) )
    {
      X <- as.matrix(cbind(X, lagMat(prmRLS$xregPrd,-prmRLS$k)))
      namesTheta <- c(namesTheta, names(prmRLS$xregPrd))
    }
  ## Remove intercept if specified
  if(!prmRLS$intercept)
    {
      X <- X[,-1]
      namesTheta <- namesTheta[-1]
    }

  ## Number of parameters
  if( is.null(ncol(X)) ){ X <- matrix(c(X,rep(NaN,n)),ncol=2) }
  p <- ncol(X)
  ## Parameter matrix
  Theta <- matrix(as.numeric(NA),nrow=n,ncol=p)
  ## The predictions
  yHat <- as.numeric(rep(NA,n))  
  ## The parameter vector
  theta <- matrix(rep(0, p), ncol=1)

  ## Start value for P
  P <- 10000 * diag(1,p)
  R <- solve(P)

  ## Only update and predict if the number of missing observations is above maxNa.
  ## NaN are allowed
  isOk <- apply( is.na(X) & !is.nan(X), 1, sum) <= prmRLS$maxNA

  ## Do it :-)
  for(t in (k+1):(n-k))
    {
      ## Parameter update, only possible if at least one measurement and y[t] is available, and isOK.
      if( isOk[t-k] & !is.na(y[t]) )
        {
          ## Only update the parameters, where measurements are available
          ok <- !is.na(X[t-k,]) 
          ## The regressor vector at time t-k
          x <- matrix(X[t-k,ok])          
          ## Update
          R[ok,ok] <- prmRLS$lambda * R[ok,ok] + x %*% t(x)	
          theta[ok] <- theta[ok] + solve(R[ok,ok],x) %*% (y[t] - t(x) %*% theta[ok])
          Theta[t,ok] <- t(theta[ok])
        }      
      if( isOk[t] )
        {
          ok <- !is.na(X[t,])
          ##Make the prediction of y, and keep it, with the regressor vector at time t
          yHat[t+k] <- t(matrix(X[t,ok])) %*% theta[ok]
        }
    }
  
  Result <- list()
  Result$yHat <- yHat
  ## Theta with names of the parameters
  Theta <- as.data.frame(Theta)
  names(Theta) <- namesTheta
  Result$Theta <- Theta
  return(Result)
}

## ## Test it
## source("~/r/functions/lagMat.R")
## n <- 100
## prmRLS <- list()
## prmRLS$k <- 1
## prmRLS$lags <- NULL
## prmRLS$xregPrd <- data.frame( rnorm(n), rnorm(n))#data.frame(pTr=Dte2$pTr, Ws=Dte2$Ws)
## prmRLS$y <- seq(0,1,len=n)*prmRLS$xregPrd[,1] + seq(1,0,len=n)*prmRLS$xregPrd[,2]
## prmRLS$xregPrd[40:60,2] <- NA
## prmRLS$xregPrd[70:80,2] <- NaN
## prmRLS$xreg <- NULL
## prmRLS$lambda <- 0.998
## prmRLS$intercept <- TRUE
## prmRLS$maxNA <- 0

## R <- rls(prmRLS)
## plot(R$Theta[,1], type="l", col=1, ylim=range(R$Theta,na.rm=TRUE))
## for(i in 2:ncol(R$Theta))
##   {
##     lines(R$Theta[,i], col=i)
##   }
## legend("topright",names(R$Theta), lty=1, col=1:ncol(R$Theta))
## prmRLS$xregPrd

## R$Theta



